@@ -1,17 +1,26 @@
+from decimal import Decimal
 import json
 import logging
 import operator
 import requests
 
+from ...metadata import Metadata
 from ...numbers import from_lovelaces
+from ...simpletypes import AssetID, Balance
 from ...transaction import Transaction
-from ...wallet import Balance
 from . import exceptions
 from . import serializers
 
 _log = logging.getLogger(__name__)
 
 
+class JSONWithDecimalEncoder(json.JSONEncoder):
+    def default(self, o):
+        if isinstance(o, Decimal):
+            return str(Decimal)
+        return super(JSONWithDecimalEncoder, self).default(o)
+
+
 class WalletREST(object):
     base_url = None
     timeout = 10
@@ -37,8 +46,10 @@ class WalletREST(object):
             url, headers=hdr, data=json.dumps(params), timeout=self.timeout
         )
         if rsp.status_code != 204:  # if content exists
-            result = rsp.json()
-            _ppresult = json.dumps(result, indent=2, sort_keys=True)
+            result = rsp.json(parse_float=Decimal)
+            _ppresult = json.dumps(
+                result, cls=JSONWithDecimalEncoder, indent=2, sort_keys=True
+            )
             _log.debug(u"Result:\n{result}".format(result=_ppresult))
         else:
             result = None
@@ -83,7 +94,7 @@ class WalletREST(object):
         return (
             1.0
             if wdata["state"]["status"] == "ready"
-            else wdata["state"]["progress"]["quantity"] / 100.0
+            else wdata["state"]["progress"]["quantity"] / Decimal(100)
         )
 
     def balance(self, wid):
@@ -102,22 +113,52 @@ class WalletREST(object):
             raise exceptions.NotSupported("Extra assets are not supported")
         assets = {}
         for ast in bdata["total"]:
-            pid = ast["policy_id"]
-            assets[pid] = assets[pid] if pid in assets else {}
-            assets[pid]["total"] = ast["quantity"]
+            aid = AssetID(ast["asset_name"], ast["policy_id"])
+            assets[aid] = assets[aid] if aid in assets else {}
+            assets[aid]["total"] = ast["quantity"]
         for ast in bdata["available"]:
-            pid = ast["policy_id"]
-            assets[pid] = assets[pid] if pid in assets else {}
-            assets[pid]["available"] = ast["quantity"]
+            aid = AssetID(ast["asset_name"], ast["policy_id"])
+            assets[aid] = assets[aid] if aid in assets else {}
+            assets[aid]["available"] = ast["quantity"]
         return {
-            pid: Balance(bal["total"], bal["available"], None) for pid, bal in assets
+            aid: Balance(bal["total"], bal["available"], None)
+            for aid, bal in assets.items()
         }
 
     def addresses(self, wid):
         adata = self.raw_request("GET", "wallets/{:s}/addresses".format(wid))
         return [(ad["id"], True if ad["state"] == "used" else False) for ad in adata]
 
-    def _txdata2tx(self, txd):
+    def used_addresses(self, wid):
+        return set(
+            [ad[0] for ad in filter(operator.itemgetter(1), self.addresses(wid))]
+        )
+
+    def _txdata2tx(self, txd, used_addresses=None):
+        inputs = (
+            [serializers.get_input(inp) for inp in txd["inputs"]]
+            if "inputs" in txd
+            else []
+        )
+        outputs = (
+            [serializers.get_output(outp) for outp in txd["outputs"]]
+            if "outputs" in txd
+            else []
+        )
+        local_inputs = set()
+        local_outputs = set()
+        if used_addresses:
+            for out in outputs:
+                if out.address and out.address in used_addresses:
+                    local_outputs.add(out)
+            for inp in inputs:
+                if inp.address and inp.address in used_addresses:
+                    local_inputs.add(inp)
+        metadata = (
+            Metadata.deserialize(txd["metadata"])
+            if txd.get("metadata", None) is not None
+            else None
+        )
         return Transaction(
             txid=txd["id"],
             gross_amount=serializers.get_amount(txd["amount"]),
@@ -131,9 +172,9 @@ class WalletREST(object):
             pending_since=serializers.get_block_position(txd["pending_since"])
             if "pending_since" in txd
             else None,
-            inputs=[serializers.get_input(inp) for inp in txd["inputs"]],
-            outputs=[serializers.get_output(outp) for outp in txd["outputs"]],
-            direction=txd["direction"],
+            inputs=inputs,
+            outputs=outputs,
+            metadata=metadata,
         )
 
     def transactions(self, wid, start=None, end=None, order="ascending"):
@@ -145,11 +186,13 @@ class WalletREST(object):
         if end is not None:
             data["end"] = end.isoformat(timespec="seconds")
         return [
-            self._txdata2tx(txd)
-            for txd in self.raw_request("GET", "wallets/{:s}/transactions".format(wid), data)
+            self._txdata2tx(txd, used_addresses=self.used_addresses(wid))
+            for txd in self.raw_request(
+                "GET", "wallets/{:s}/transactions".format(wid), data
+            )
         ]
 
-    def transfer(self, wid, destinations, ttl, passphrase):
+    def transfer(self, wid, destinations, metadata, ttl, passphrase):
         data = {
             "passphrase": passphrase,
             "payments": [
@@ -160,8 +203,13 @@ class WalletREST(object):
                 for (address, amount) in destinations
             ],
         }
+        if metadata is not None:
+            if not isinstance(metadata, Metadata):
+                metadata = Metadata(metadata.items())
+            data["metadata"] = metadata.serialize()
         if ttl is not None:
             data["time_to_live"] = serializers.store_interval(ttl)
-        return self._txdata2tx(
-            self.raw_request("POST", "wallets/{:s}/transactions".format(wid), data)
-        )
+        # NOTE: the order of the following two requests is important
+        txd = self.raw_request("POST", "wallets/{:s}/transactions".format(wid), data)
+        used_addresses = self.used_addresses(wid)
+        return self._txdata2tx(txd, used_addresses)
