@@ -1,6 +1,3 @@
-import collections
-from datetime import timezone
-
 from .address import Address
 from . import exceptions
 
@@ -31,13 +28,6 @@ class WalletService(object):
         return self.backend.create_wallet(name, mnemonic, passphrase, mnemonic_2f)
 
 
-Balance = collections.namedtuple("Balance", ["total", "available", "reward"])
-Balance.__doc__ = "Represents a balance of asset, including total, principal and reward"
-Balance.total.__doc__ = "The total balance"
-Balance.available.__doc__ = "The principal, i.e. the total minus staking rewards"
-Balance.reward.__doc__ = "The staking rewards (interest)"
-
-
 class Wallet(object):
     """
     Represents a single wallet. Allows for browsing the history, checking balance and spending
@@ -82,13 +72,15 @@ class Wallet(object):
     def delete(self):
         return self.backend.delete_wallet(self.wid)
 
-    def transactions(self): #, start=None, end=None, order="ascending"):
+    def transactions(self):  # , start=None, end=None, order="ascending"):
         # WARNING: parameters don't really work; this is a known bug
-        #if start:
+        # if start:
         #    start = start.astimezone(tz=timezone.utc)
-        #if end:
+        # if end:
         #    end = end.astimezone(tz=timezone.utc)
-        return self.backend.transactions(self.wid) #, start=start, end=end, order=order)
+        return self.backend.transactions(
+            self.wid
+        )  # , start=start, end=end, order=order)
 
     def _resolve_passphrase(self, passphrase):
         passphrase = passphrase or self.passphrase
@@ -98,12 +90,14 @@ class Wallet(object):
             )
         return passphrase
 
-    def transfer(self, address, amount, ttl=None, passphrase=None):
+    def transfer(self, address, amount, metadata=None, ttl=None, passphrase=None):
         """
         Sends a transfer from the wallet. Returns the resulting transaction.
 
         :param address: destination :class:`Address <cardano.address.Address>` or subtype
         :param amount: amount to send
+        :param metadata: metadata to be sent, as :class:`Metadata <cardano.metadata.Metadata>`
+                    instance od ``dict`` mapping ``int`` keys to values of acceptable types
         :param ttl: Time To Live in seconds. After TTL has lapsed the nodes give up on broadcasting
                     the transaction. Leave `None` to use the default value.
         :param passphrase: the passphrase to the wallet. It takes precedence over `self.passphrase`
@@ -115,16 +109,19 @@ class Wallet(object):
         return self.backend.transfer(
             self.wid,
             ((address, amount),),
-            ttl=ttl,
-            passphrase=self._resolve_passphrase(passphrase),
+            metadata,
+            ttl,
+            self._resolve_passphrase(passphrase),
         )
 
-    def transfer_multiple(self, destinations, amount, ttl=None, passphrase=None):
+    def transfer_multiple(self, destinations, metadata=None, ttl=None, passphrase=None):
         """
         Sends multiple transfers from the wallet. Returns the resulting transaction.
 
         :param destinations: a list of :class:`Address <cardano.address.Address>` and amount
-                    pairs: [(address, amount), ...]
+                    pairs ``[(address, amount), ...]``
+        :param metadata: metadata to be sent, as :class:`Metadata <cardano.metadata.Metadata>`
+                    instance od ``dict`` mapping ``int`` keys to values of acceptable types
         :param ttl: Time To Live in seconds. After TTL has lapsed the nodes give up on broadcasting
                     the transaction. Leave `None` to use the default value.
         :param passphrase: the passphrase to the wallet. It takes precedence over `self.passphrase`
@@ -133,14 +130,10 @@ class Wallet(object):
                     exception will be raised.
         :rtype: :class:`Transaction <cardano.transaction.Transaction>`
         """
-        passphrase = passphrase or self.passphrase
-        if passphrase is None:
-            raise exceptions.MissingPassphrase(
-                "Cannot transfer from wallet without the passphrase"
-            )
         return self.backend.transfer(
             self.wid,
             destinations,
-            ttl=ttl,
-            passphrase=self._resolve_passphrase(passphrase),
+            metadata,
+            ttl,
+            self._resolve_passphrase(passphrase),
         )
